==> oembed_lazyload-2.0.x-dev.tar.gz: oembed_lazyload/modules/oembed_lazyload_youtube/src/Plugin/oembed_lazyload/ProviderEnhancer/YoutubeEnhancer.php <==
<?php

namespace Drupal\oembed_lazyload_youtube\Plugin\oembed_lazyload\ProviderEnhancer;

use Drupal\media\OEmbed\Resource;
use Drupal\oembed_lazyload\ProviderEnhancerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * An implementation that enhances YouTube content.
 *
 * @ProviderEnhancer(
 *   id = "youtube",
 *   label = "YouTube",
 *   providers = {
 *     "YouTube"
 *   }
 * )
 */
class YoutubeEnhancer extends ProviderEnhancerBase {

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The request stack service.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->configFactory = $container->get('config.factory');
    $instance->requestStack = $container->get('request_stack');
    return $instance;
  }

  /**
   * Attempts to parse out an embed code from the provided url.
   *
   * @param string $url
   *   The url to attempt to parse.
   *
   * @return false|string|null
   *   The embed code, or a falsy value if the embed code cannot be parsed.
   */
  protected function getEmbedCode($url) {

    $embed_code = NULL;
    if (strpos($url, 'youtube.com/watch') !== FALSE) {
      $embed_code = substr($url, strpos($url, '?v=') + 3);
    }
    elseif (strpos($url, 'youtube.com/v/') !== FALSE) {
      $embed_code = substr($url, strpos($url, '/v/') + 3);
    }
    elseif (strpos($url, 'youtu.be/') !== FALSE) {
      $embed_code = substr($url, strpos($url, 'youtu.be/') + 9);
    }
    elseif (strpos($url, 'youtube.com/playlist?list=') !== FALSE) {
      $embed_code = substr($url, strpos($url, 'youtube.com/playlist?list=') + 26);
    }
    elseif (strpos($url, 'youtube.com/shorts/') !== FALSE) {
      $embed_code = substr($url, strpos($url, 'youtube.com/shorts/') + 19);
    }

    // Cut off any query parameters and fragments.
    return preg_replace('/[?&#].*/', '', $embed_code);
  }

  /**
   * {@inheritdoc}
   */
  public function getPlaceholder($url, Resource $resource, array $settings) {
    $placeholder = parent::getPlaceholder($url, $resource, $settings);
    if ($embed_code = $this->getEmbedCode($url)) {
      $third_party_settings = [
        'embed_code' => $embed_code,
      ];
      $placeholder['#third_party_settings'] = $third_party_settings;
    }
    return $placeholder;
  }

  /**
   * {@inheritdoc}
   */
  public function getLibraries() {
    $libraries = parent::getLibraries();
    $libraries[] = 'oembed_lazyload_youtube/youtube';
    return $libraries;
  }

  /**
   * {@inheritdoc}
   *
   * The $options array may contain:
   *   - modestbranding.
   */
  public function alterOembedResponse($markup, array $options = []) {
    $replace = '?feature=oembed';

    if (isset($options['autoplay']) && $options['autoplay'] === '1') {
      $replace .= '&autoplay=1';
    }

    if (isset($options['modestbranding']) && $options['modestbranding'] === '1') {
      $replace .= '&modestbranding=1';
    }

    if (isset($options['enablejsapi']) && $options['enablejsapi'] === '1') {
      $replace .= '&enablejsapi=1';

      if (isset($options['origin']) && $options['origin'] === '1') {
        $media_settings = $this->configFactory->get('media.settings');
        $iframe_domain = $media_settings->get('iframe_domain');
        if (empty($iframe_domain)) {
          $request = $this->requestStack->getCurrentRequest();
          if ($request) {
            $iframe_domain = $request->getSchemeAndHttpHost();
          }
        }
        if (!empty($iframe_domain)) {
          $replace .= "&origin=$iframe_domain";
        }
      }
    }

    if (isset($options['hideinfo']) && $options['hideinfo'] === '1') {
      $replace .= '&showinfo=0';
    }

    if (isset($options['rel']) && $options['rel'] === '1') {
      $replace .= '&rel=0';
    }

    return str_replace('?feature=oembed', $replace, $markup);
  }

}

==> oembed_lazyload-2.1.x-dev.tar.gz: oembed_lazyload/modules/oembed_lazyload_youtube/src/Plugin/oembed_lazyload/ProviderEnhancer/YoutubeEnhancer.php <==
<?php

namespace Drupal\oembed_lazyload_youtube\Plugin\oembed_lazyload\ProviderEnhancer;

use Drupal\media\OEmbed\Resource;
use Drupal\oembed_lazyload\ProviderEnhancerBase;
use GuzzleHttp\Exception\ClientException;
use GuzzleHttp\RequestOptions;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * An implementation that enhances YouTube content.
 *
 * @ProviderEnhancer(
 *   id = "youtube",
 *   label = "YouTube",
 *   providers = {
 *     "YouTube"
 *   }
 * )
 */
class YoutubeEnhancer extends ProviderEnhancerBase {

  /**
   * All possible thumbnails that may be available.
   */
  protected const THUMBNAIL_RESOLUTIONS = [
    'maxresdefault' => ['width' => 1920, 'height' => 1080],
    'hq720' => ['width' => 1280, 'height' => 720],
    'sddefault' => ['width' => 640, 'height' => 480],
    'hqdefault' => ['width' => 480, 'height' => 360],
    'mqdefault' => ['width' => 320, 'height' => 180],
    'default' => ['width' => 120, 'height' => 90],
  ];

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The request stack service.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The http client service.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->configFactory = $container->get('config.factory');
    $instance->requestStack = $container->get('request_stack');
    $instance->httpClient = $container->get('http_client');
    return $instance;
  }

  /**
   * Attempts to parse out an embed code from the provided url.
   *
   * @param string $url
   *   The url to attempt to parse.
   *
   * @return false|string|null
   *   The embed code, or a falsy value if the embed code cannot be parsed.
   *
   * @deprecated in oembed_lazyload:2.1.0 and is removed from oembed_lazyload:3.0.0. Switch to using regex based code extraction instead.
   *
   * @see https://www.drupal.org/node/3415047
   */
  protected function getEmbedCode($url) {

    $embed_code = NULL;
    if (strpos($url, 'youtube.com/watch') !== FALSE) {
      $embed_code = substr($url, strpos($url, '?v=') + 3);
    }
    elseif (strpos($url, 'youtube.com/v/') !== FALSE) {
      $embed_code = substr($url, strpos($url, '/v/') + 3);
    }
    elseif (strpos($url, 'youtu.be/') !== FALSE) {
      $embed_code = substr($url, strpos($url, 'youtu.be/') + 9);
    }
    elseif (strpos($url, 'youtube.com/playlist?list=') !== FALSE) {
      $embed_code = substr($url, strpos($url, 'youtube.com/playlist?list=') + 26);
    }
    elseif (strpos($url, 'youtube.com/shorts/') !== FALSE) {
      $embed_code = substr($url, strpos($url, 'youtube.com/shorts/') + 19);
    }

    // Cut off any query parameters and fragments.
    return preg_replace('/[?&#].*/', '', $embed_code);
  }

  /**
   * {@inheritdoc}
   */
  public function getPlaceholder($url, Resource $resource, array $settings) {
    $placeholder = parent::getPlaceholder($url, $resource, $settings);

    if ($settings['placeholder_asset_type'] === 'remote_asset' && $embed_code = $this->getEmbedCode($url)) {
      $third_party_settings = [
        'embed_code' => $embed_code,
      ];
      $placeholder['#third_party_settings'] = $third_party_settings;
    }
    return $placeholder;
  }

  /**
   * {@inheritdoc}
   */
  public function getLibraries() {
    $libraries = parent::getLibraries();
    $libraries[] = 'oembed_lazyload_youtube/youtube';
    return $libraries;
  }

  /**
   * {@inheritdoc}
   *
   * The $options array may contain:
   *   - modestbranding.
   */
  public function alterOembedResponse($markup, array $options = []) {
    $replace = '?feature=oembed';

    if (isset($options['autoplay']) && $options['autoplay'] === '1') {
      $replace .= '&autoplay=1';
    }

    if (isset($options['modestbranding']) && $options['modestbranding'] === '1') {
      $replace .= '&modestbranding=1';
    }

    if (isset($options['enablejsapi']) && $options['enablejsapi'] === '1') {
      $replace .= '&enablejsapi=1';

      if (isset($options['origin']) && $options['origin'] === '1') {
        $media_settings = $this->configFactory->get('media.settings');
        $iframe_domain = $media_settings->get('iframe_domain');
        if (empty($iframe_domain)) {
          $request = $this->requestStack->getCurrentRequest();
          if ($request) {
            $iframe_domain = $request->getSchemeAndHttpHost();
          }
        }
        if (!empty($iframe_domain)) {
          $replace .= "&origin=$iframe_domain";
        }
      }
    }

    if (isset($options['hideinfo']) && $options['hideinfo'] === '1') {
      $replace .= '&showinfo=0';
    }

    if (isset($options['rel']) && $options['rel'] === '1') {
      $replace .= '&rel=0';
    }

    return str_replace('?feature=oembed', $replace, $markup);
  }

  /**
   * {@inheritdoc}
   */
  public function alterOembedMetadata(array &$data, $url) {
    parent::alterOembedMetadata($data, $url);
    $config = $this->configFactory->get('oembed_lazyload_youtube.settings');

    if ($config->get('resource_fetcher_thumbnail_strategy') === 'prefer_max_resolution') {
      $matches = [];
      if (preg_match('/.*\/([\w-]+)\/.*\.(jpg|webp)/', $data['thumbnail_url'], $matches)) {
        $embed_code = $matches[1];
        // Get the highest resolution thumbnail available, if possible.
        foreach (static::THUMBNAIL_RESOLUTIONS as $format => $resolution) {
          $thumbnail_url = "https://i.ytimg.com/vi_webp/$embed_code/$format.webp";
          try {
            $response = $this->httpClient->request(
              'get',
              $thumbnail_url,
              [RequestOptions::TIMEOUT => 5],
            );
            if ($response->getStatusCode() === 200) {
              $data['thumbnail_url'] = $thumbnail_url;
              $data['thumbnail_width'] = $resolution['width'];
              $data['thumbnail_height'] = $resolution['height'];
              break;
            }
          }
          catch (ClientException $e) {
            // Probably a 404 - ignore it, as it's expected.
          }
        }
      }
    }
  }

}
